
function [features,ij] = edge_features(A, varargin)
	% Extract features for all edges in a graph that is given as a sparse matrix.
	% 
	% Usage:
	%   [X,ij] = edge_features(A, varargin)
	% Input:
	%   A = a sparse n*n matrix
	%   varargin = a list of name,value pairs
	% Possible parameters in varargin:
	%   'path_length'  = count paths up to and including this length (default: 2, i.e. triangles)
	%   'degrees'      = include features for degree of adjacent vertices (default: true)
	%   'bias'         = include bias term (default: true)
	%   'poly'         = generate polynomial features of given degree (default: 1)
	%   'node_average' = include average of other features over edges incident on the two endpoints (default: true)
	% Output:
	%   E = endpoints of edges, E(:,1) = first vertex, E(:,2) = second vertex, E(:,3) = index in sparse matrix
	%   X = features for each edge
	%
	% Currently the extracted features are:
	%   * a bias term
	%   * node degree of adjacent vertices (sorted)
	%   * number of paths of length 2 (triangles) between the endpoints of the edge
	
	% Parameters
	params = struct(varargin{:});
	if ~isfield(params,'path_length'), params.path_length = 2; end
	if ~isfield(params,'degrees'),     params.degrees = true; end
	if ~isfield(params,'degrees_minmax'),params.degrees_minmax = true; end
	if ~isfield(params,'bias'),        params.bias = true; end
	if ~isfield(params,'poly'),        params.poly = 1; end
	if ~isfield(params,'node_average'),params.node_average = 0; end
	if ~isfield(params,'node_average_minmax'),params.node_average_minmax = true; end
	if ~isfield(params,'node_quantiles'),params.node_quantiles = 0; end
	
	[i,j] = find(A);
	
	features = zeros(numel(i),0);
	
	% min/max d
	if params.degrees && ~isequal(params.degrees,'none')
		d = full(sum(A,2));
		d1 = d(i);
		d2 = d(j);
		if params.degrees_minmax
			features = [features, min(d1,d2), max(d1,d2)];
		else
			features = [features, d1+d2];
		end
	end
	
	al=A;
	for l = 2:params.path_length
		al = A * al;
		paths_l = index_sparse(al,i,j);
		features = [features,paths_l];
	end
	
	% avergage feature values of nodes
	for rep = 1:params.node_average
		f = zeros(length(A),size(features,2));
		d = accumarray(i,ones(size(i)),[length(A),1]);
		for k=1:size(features,2)
			sum_fk = accumarray(i,features(:,k),[length(A),1]);
			f(:,k) = sum_fk ./ d;
		end
		% min and max for each edge?
		% or based on node degree?
		f1 = f(i,:);
		f2 = f(j,:);
		if params.node_average_minmax
			features = [features,min(f1,f2),max(f1,f2)];
		else
			features = [features,f1+f2];
		end
	end
	
	% quantile feature values of nodes
	if params.node_quantiles
		qs = (1:params.node_quantiles) / (params.node_quantiles+1);
		TODO;
		%grouped = ;
		%f = cellfun(vec(),);
	end
	
	% bias term
	if params.bias
		bias = vec(ones(size(i)));
		features = [bias, features];
	end
	
	if params.poly > 1
		features = feature_transform(features,params.poly);
	end
end

function X = vec(X)
	% Convert a matrix or vector into a row vector
	% 
	% This is an octave builtin, but it doesn't exist in matlab.
	X = X(:);
end
