
function [d,A] = find_communities_with_edge_classifier(A,X,w, varargin)
	% Find communities by applying a classifier to the edges E,
	% and then finding the connected component of the reduced graph.
	% 
	% Usage:
	%   d = find_communities_edge_classifier(A,X,w, varargin)
	% Input:
	%   A = sparse matrix representing the graph,             size: n*n
	%   X = for each edge (row) a list of features (columns), size: m*f
	%       edges are ordered as by find(A)
	%   w = weight vector for linear classifier,              size: f*1
	%   varargin = optional key/value parameters
	% Possible parameters:
	%   'min_edge_per_node',k  = ensure that at least k edges remain per node (default: 0)
	% Output:
	%   d = labeling of communities / connected compnents,    size: n
	
	% Parameters
	params = struct(varargin{:});
	if ~isfield(params,'min_edge_per_node'), params.min_edge_per_node = 0; end
	if ~isfield(params,'remove_fraction'), params.remove_fraction = 1; end
	
	% Apply classifer
	if params.remove_fraction < 1
		while 1
			B = sparsify_graph_with_classifier(A,X,w,params);
			if nnz(A) == nnz(B), break; end
			A = B;
			% Recalculate the features, this is a bit of a hack
			X = edge_features(A,varargin{:});
		end
	else
		A = sparsify_graph_with_classifier(A,X,w,params);
	end
	
	% Return its connected components
	d = connected_components(A);
end

function [B,th] = sparsify_graph_with_classifier(A,X,w,params)
	% Edges to keep according to classifier
	[i,j] = find(A);
	xw = X*(w(:));
	
	if params.remove_fraction < 1
		% remove at most 'remove_fraction' of all edges
		th = quantile(xw, params.remove_fraction);
		th = min(0,th)
	else
		th = 0;
	end
	keep = xw > th;
	
	% Keep a minimum number of edges for each node
	if params.min_edge_per_node == 1
		if 0
			best_out = accumarray(i,xw,[length(A),1],@max,-Inf);
			keep(xw == best_out(i)) = 1;
		elseif 1
			best_out = accumarray_max(i,xw,length(A));
			keep(xw == best_out(i)) = 1;
		end
	elseif params.min_edge_per_node > 0
		if 0
			%[val,ind] = sort(xw, 'descend');
			for ii = 1:length(A)
				edges_from_ii = find(i == ii & j ~= ii);
				if params.min_edge_per_node == 1
					[score,idx] = max(xw(edges_from_ii));
				else
					[score,idx] = sort(xw(edges_from_ii), 'descend');
					if length(idx) > params.min_edge_per_node
						idx = idx(1:params.min_edge_per_node);
					end
				end
				keep( edges_from_ii(idx) ) = 1;
			end
		else
			for iter=1:params.min_edge_per_node
				if iter == 1
					lower = 1:size(i);
				else
					lower = xw < best_out(i);
				end
				best_out = accumarray_max(i(lower)',xw(lower)',length(A));
				keep(xw == best_out(i)) = 1;
			end
		end
	end
	
	% convert to a sparse matrix
	B = sparse(i(keep),j(keep),ones(nnz(keep),1),length(A),length(A));
	
	% Make sure the graph is symmetric
	B = B | B';
end

function idx = maxIdx(idxes,vals)
	vs = vals(idxes);
	[mv,mi] = max(vs);
	idx = todo;
end
