
function nmi = normalized_mutual_information(la,lb, normalize_method)
	% Calculate the normalized mutual information of two labelings.
	% 
	% Usage:
	%   normalized_mutual_information(la,lb [,method])
	% Parameter:
	%   la,lb = two labelings, of the same length
	%   method = normalization method, one of
	%        'mean' (default),
	%        'max'
	%        'none'
	%
	% Calculates:
	%   method 'mean':    NI(a,b) = I(a,b) / ((H(a) + H(b))/2)
	%   method 'max':     NI(a,b) = I(a,b) / max(H(a), H(b))
	%   method 'fst':     NI(a,b) = I(a,b) / H(a)
	%   method 'snd':     NI(a,b) = I(a,b) / H(b)
	%   method 'var':     NI(a,b) = 1 - (H(a|b)/H(a) + H(b|a)/H(b))/2 = (I(a,b) / H(a) + I(a,b) / H(b)) / 2
	%   method 'entropy': NI(a,b) = H(a,b)
	%   method 'none':    NI(a,b) = I(a,b)
	
	
	%   H(a) = sum{x} P(a==x)*log(P(a==x)) % entropy
	%   I(a,b) = sum{x,y} P(a==x&&b==y)*log(P(a==x&&b==y)/(P(a==x)*P(b==y)))
	%   P(a==x) = count(a==c) / length(a)
	
	la = la(:); lb = lb(:);
	
	n   = length(la);
	ca  = replace_counts(la);
	cb  = replace_counts(lb);
	cab = replace_counts(la + lb*(max(la)-min(la)+1));
	entropy_a = sum(-log(ca ./ n));
	entropy_b = sum(-log(cb ./ n));
	mutual_info = sum(log(n .* cab ./ (ca .* cb)));
	if nargin < 3 || isequal(normalize_method,'mean')
		nmi = 2 * mutual_info / max(1e-6, entropy_a + entropy_b);
	elseif isequal(normalize_method,'max')
		nmi = mutual_info / max(1e-6, max(entropy_a, entropy_b));
	elseif isequal(normalize_method,'fst')
		nmi = mutual_info / entropy_a;
	elseif isequal(normalize_method,'snd')
		nmi = mutual_info / entropy_b;
	elseif isequal(normalize_method,'entropy')
		nmi = entropy_a + entropy_b - mutual_info;
	elseif isequal(normalize_method,'var')
		% Used by Lancichinetti et.al. 2009
		h_a_given_b = entropy_a - mutual_info;
		h_b_given_a = entropy_b - mutual_info;
		nmi = 1 - ((h_a_given_b / entropy_a) + (h_b_given_a / entropy_b)) / 2;
	elseif isequal(normalize_method,'none')
		nmi = mutual_info;
	else
		error('normalized_mutual_information: invalid normalization method');
	end
end

