function out = run_benchmarks_for_classifier(classifier, varargin)
	% Run LFR benchmarks with a given classifier
	% Usage:
	%   out = run_benchmarks_for_classifier(classifer, ...)
	% Arguments:
	%   classifier = either a weight vector, or a learning method
	%   ... = extra options to pass to edge_features, and to classifier
	% Result:
	%   a structure containing statistics
	
	% parameters (?)
	params = struct(varargin{:});
	if ~isfield(params,'add_community_feature'), params.add_community_feature = false; end
	if ~isfield(params,'include_gn'), params.include_gn = true; end
	if ~isfield(params,'include_1k'), params.include_1k = true; end
	if ~isfield(params,'include_5k'), params.include_5k = true; end
	if ~isfield(params,'num_copies'), params.num_copies = 5; end
	if ~isfield(params,'mu_min'),  params.mu_min  = 0.0; end
	if ~isfield(params,'mu_max'),  params.mu_max  = 0.85; end
	if ~isfield(params,'mu_step'), params.mu_step = 0.05; end
	
	% which benchmarks to run?
	benchmarks = cell();
	if params.include_1k
		benchmarks{end+1} = {'Small 1000', '-N 1000 -t1 2 -t2 1 -k 20 -maxk 50 -minc 10 -maxc 50'};
		benchmarks{end+1} = {'Big 1000',   '-N 1000 -t1 2 -t2 1 -k 20 -maxk 50 -minc 20 -maxc 100'};
	end
	if params.include_5k
		benchmarks{end+1} = {'Small 5000', '-N 5000 -t1 2 -t2 1 -k 20 -maxk 50 -minc 10 -maxc 50'};
		benchmarks{end+1} = {'Big 5000',   '-N 5000 -t1 2 -t2 1 -k 20 -maxk 50 -minc 20 -maxc 100'};
	end
	if params.include_gn
		benchmarks{end+1} = {'GN',         '-N 128 -k 16 -maxk 16 -minc 32 -maxc 32'};
	end
	% other options
	mus = params.mu_min:params.mu_step:params.mu_max;
	
	% outputs
	out.mus = mus;
	out.benchmarks = benchmarks;
	out.names = cellfun(@(x)x{1},benchmarks,'UniformOutput',false);
	
	out.nmi = {};
	out.nmi_max = {};
	out.num = {};
	out.purity = {};
	out.fraction_kept = {};
	out.edge_precision = {};
	out.edge_recall = {};
	out.edge_accuracy = {};
	
	more off;
	
	% filename to save to disk, just to be sure
	if isfield(params,'filename')
		fn  = [params.filename '.txt'];
		fn2 = [params.filename '.tex'];
	else
		date = datestr(now,'yyyymmddTHHMMSS');
		mkdir('../out');
		fn = sprintf('../out/benchmarks_for_%s_%s.txt',function_to_string(classifier),date);
		fn2 = sprintf('../out/benchmarks_for_%s_%s.tex',function_to_string(classifier),date);
	end
	printf("Results will be saved as %s\n",fn);
	
	% run
	for j = 1:params.num_copies
		printf("Iteration %d of %d\n",j,params.num_copies);
		for i_bench = 1:numel(benchmarks)
			for i_mu = 1:numel(mus)
				% parameters for generator
				mu = mus(i_mu);
				data_params = sprintf('%s -mu %f',benchmarks{i_bench}{2},mu);
				
				% Generate data
				[A,c] = random_powerlaw_graph(data_params,j);
				X = edge_features(A, varargin{:});
				
				% Is the classifier a function? Then train it
				if isnumeric(classifier)
					w = classifier;
				else
					[A_tr,c_tr] = random_powerlaw_graph(data_params,j+12345);
					[A_va,c_va] = random_powerlaw_graph(data_params,j+45678);
					X_tr = edge_features(A_tr, varargin{:});
					X_va = edge_features(A_va, varargin{:});
					w = classifier(A_tr,c_tr,X_tr, A_va,c_va,X_va, varargin{:});
				end
				
				% Apply method
				[d,B] = find_communities_with_edge_classifier(A,X,w, 'min_edge_per_node',1);
				
				% Results
				out.nmi          {i_bench,i_mu}(j) = normalized_mutual_information(c,d);
				out.nmi_max      {i_bench,i_mu}(j) = normalized_mutual_information(c,d,'max');
				out.num          {i_bench,i_mu}(j) = numel(unique(d));
				out.purity       {i_bench,i_mu}(j) = cluster_purity(c,d);
				out.fraction_kept{i_bench,i_mu}(j) = nnz(B) / nnz(A);
				% as edge classifier
				[posB] = edge_accuracy(B,c);
				[posA] = edge_accuracy(A,c);
				trueNeg = nnz(A) - posA - (nnz(B) - posB);
				out.edge_precision{i_bench,i_mu}(j) = posB / nnz(B);
				out.edge_recall   {i_bench,i_mu}(j) = posB / posA;
				out.edge_accuracy {i_bench,i_mu}(j) = (posB + trueNeg) / nnz(A);
				
				printf(" bench = %s, mu = %f  nmi = %f\n",benchmarks{i_bench}{1},mu,out.nmi{i_bench,i_mu}(j));
			end
			
			print_benchmark_results(fn2, out.names,mus,out.nmi);
		end
		
		% Plot
		errorbar_plot(mus,cellfun(@mean,out.nmi),cellfun(@std,out.nmi));
		legend(out.names);
		sleep(0.01); % give octave a chance to draw the legend
		
		% Save
		save(fn,'classifier','out');
	end
	
	% Export to .tex
	print_benchmark_results(stdout, out.names,mus,out.nmi);
	print_benchmark_results(fn2, out.names,mus,out.nmi);
end

function [pos,neg] = edge_accuracy(A,c)
	[i,j] = find(A);
	pos = nnz(c(i) == c(j));
	neg = nnz(c(i) ~= c(j));
end
