
function w = train_edgewise_svm_liblinear(A,X,c, varargin)
	% Learn a linear classifier, based on the path edge-wise loss.
	%
	% Usage:
	%   w = learn_edge_svm_sgd(A,X,c, varargin)
	% Input:
	%   A = sparse matrix representing the graph,             size: n*n
	%   X = for each edge (row) a list of features (columns), size: m*f
	%       edges are ordered as by find(A)
	%   c = for each vertex, a community label, or -1,        size: n
	%   varargin = a list of name,value pairs
	% Possible parameters:
	%   'lambda'         = regularization parameter (default: 0)
	%   'weight_same'    = weight for inside community edges (default: 1)
	%   'weight_diff'    = weight for between community edges (default: 1)
	%   'standardize'    = standardize the input vectors? (default: 0)
	% Output:
	%   w = weight vector for linear classifier,              size: f*1
	
	% Parameters
	params = struct(varargin{:});
	if ~isfield(params,'lambda'),         params.lambda         = 1; end
	if ~isfield(params,'weight_same'),    params.weight_same    = 1; end
	if ~isfield(params,'weight_diff'),    params.weight_diff    = 1; end
	if ~isfield(params,'standardize'),    params.standardize    = 0; end
	if ~isfield(params,'svm_mode'),       params.svm_mode       = 2; end
	
	% Prepare data: standardize. Assume that the first term is bias term
	if params.standardize
		if any(X(:,1) ~= 1)
			X = [ones(size(X,1),1),X];
		end
		meanX = [0, mean(X(:,2:end))];
		stdX = max(1e-6, [1, std(X(:,2:end))]);
		X = bsxfun(@rdivide,bsxfun(@minus,X,meanX),stdX);
	end
	
	% Prepare data: Y
	Y = edge_labels(A,c);
	
	% remove zero-weight stuff
	X(Y == 0,:) = [];
	Y(Y == 0,:) = [];
	
	% parameters for libSVM
	C = 1 / (params.lambda);
	svm_opts = sprintf('-s %d -q -c %f',params.svm_mode,C);
	if any(Y== 1) svm_opts = sprintf('%s -w1 %f',svm_opts, params.weight_same); end
	if any(Y==-1) svm_opts = sprintf('%s -w-1 %f',svm_opts, params.weight_diff); end
	
	% Train svm
	model = train(Y+0,sparse(X),svm_opts);
	w = model.w';
	if ~model.Label(1), w=-w; end;
	
	% Compensate w for non-standardized data
	if params.standardize
		w = w ./ stdX';
		w(1) -= dot(meanX,w);
	end
end
